@@ -24,6 +24,8 @@
 #include "config.h"
 #endif
 
+#include <unistd.h>
+#include <sys/types.h>
 #include <stdlib.h>
 #include <signal.h>
 #include <grp.h>
@@ -38,11 +40,13 @@
 #include "update-notifier.h"
 #include "update.h"
 #include "hooks.h"
-#include "gdu.h"
+#include "cdroms.h"
 #include "reboot.h"
-#include "firmware.h"
+#include "uevent.h"
 #include "crash.h"
 #include "avahi.h"
+#include "release.h"
+#include "trayappletui.h"
 
 /* some prototypes */
 extern gboolean up_get_clipboard (void);
@@ -59,7 +63,7 @@ gboolean update_timer_finished(gpointer data);
 // causes the applet to "ungray"
 #define TIMEOUT_APT_RUN 600 /* 600 sec */
 
-// force startup even if the user is not in the admin group
+// force startup even if the user is not in the sudo/admin group
 static gboolean FORCE_START=FALSE;
 
 // force gksu for all menu actions
@@ -68,11 +72,19 @@ static gboolean FORCE_GKSU=FALSE;
 // delay on startup (to make session startup faster)
 static int STARTUP_DELAY=1;
 
+// make the release upgrader check for a devel release
+gboolean DEVEL_RELEASE=FALSE;
+
+// force release check
+static gboolean FORCE_RELEASE_CHECK=FALSE;
+
 // global debug options
 int HOOK_DEBUG = 0;
 int UPDATE_DEBUG = 0;
 int INOTIFY_DEBUG = 0;
-int FIRMWARE_DEBUG = 0;
+int UEVENT_DEBUG = 0;
+int RELEASE_DEBUG = 0;
+int MISC_DEBUG = 0;
 
 // logging stuff
 static void debug_log_handler (const gchar   *log_domain,
@@ -80,14 +92,28 @@ static void debug_log_handler (const gchar   *log_domain,
                               const gchar   *message,
                               gpointer       user_data)
 {
-   if (HOOK_DEBUG > 0 && strcmp(log_domain, "hooks") == 0)
-       g_log_default_handler (log_domain, log_level, message, user_data);
-   if (UPDATE_DEBUG > 0 && strcmp(log_domain, "update") == 0)
-       g_log_default_handler (log_domain, log_level, message, user_data);
-   if (INOTIFY_DEBUG > 0 && strcmp(log_domain, "inotify") == 0)
-       g_log_default_handler (log_domain, log_level, message, user_data);
-   if (FIRMWARE_DEBUG > 0 && strcmp(log_domain, "firmware") == 0)
-       g_log_default_handler (log_domain, log_level, message, user_data);
+   if (log_domain && strcmp(log_domain, "hooks") == 0) {
+      if (HOOK_DEBUG > 0)
+         g_log_default_handler (log_domain, log_level, message, user_data);
+   }
+   else if (log_domain && strcmp(log_domain, "update") == 0) {
+      if (UPDATE_DEBUG > 0)
+         g_log_default_handler (log_domain, log_level, message, user_data);
+   }
+   else if (log_domain && strcmp(log_domain, "inotify") == 0) {
+      if (INOTIFY_DEBUG > 0) 
+          g_log_default_handler (log_domain, log_level, message, user_data);
+   }
+   else if (log_domain && strcmp(log_domain, "uevent") == 0) {
+      if (UEVENT_DEBUG > 0)
+         g_log_default_handler (log_domain, log_level, message, user_data);
+   }
+   else if (log_domain && strcmp(log_domain, "release") == 0) {
+      if (RELEASE_DEBUG > 0)
+         g_log_default_handler (log_domain, log_level, message, user_data);
+   }
+   else if (MISC_DEBUG > 0)
+      g_log_default_handler (log_domain, log_level, message, user_data);
 }
 
 static inline void
@@ -99,6 +125,32 @@ g_debug_inotify(const char *msg, ...)
    va_end(va);
 }
 
+// launch async scripts one after another
+static void start_next_script(GPid pid,
+                              gint status,
+                              GSList *remaining_scripts_to_run)
+{
+    if(remaining_scripts_to_run) {
+        gboolean     ret = False;
+        gchar*       full_path = remaining_scripts_to_run->data;
+        gchar       *argv[] = { full_path, NULL };
+
+        g_debug_inotify ("executing: %s\n", full_path);
+        if (g_file_test(full_path, G_FILE_TEST_IS_EXECUTABLE ))
+            ret = g_spawn_async("/", argv, NULL, G_SPAWN_DO_NOT_REAP_CHILD, NULL, NULL, &pid, NULL);
+        remaining_scripts_to_run = g_slist_remove(remaining_scripts_to_run, full_path);
+        g_free(full_path);
+        if(!ret)
+        {
+            g_warning("script execution failed or not executable");
+            start_next_script(0, 0, remaining_scripts_to_run);
+            return;
+        }
+        g_child_watch_add(pid, (GChildWatchFunc)start_next_script, remaining_scripts_to_run);
+    }
+
+}
+
 void invoke(const gchar *cmd, const gchar *desktop, gboolean with_gksu)
 {
    GdkAppLaunchContext *context;
@@ -120,7 +172,7 @@ void invoke(const gchar *cmd, const gchar *desktop, gboolean with_gksu)
 
    // normal launch
    context = gdk_app_launch_context_new ();
-   guint32 timestamp =  gdk_x11_get_server_time (w->window);
+   guint32 timestamp =  gdk_x11_get_server_time (gtk_widget_get_window (w));
    appinfo = g_app_info_create_from_commandline(cmd, 
 						cmd, 
 						G_APP_INFO_CREATE_NONE,
@@ -136,7 +188,7 @@ void invoke(const gchar *cmd, const gchar *desktop, gboolean with_gksu)
 void
 invoke_with_gksu(const gchar *cmd, const gchar *descr, gboolean whole_message)
 {
-        //g_print("invoke_update_manager ()\n");
+        g_debug("invoke_update_manager ()\n");
         gchar *argv[5];
 	argv[0] = "/usr/bin/gksu";
 	argv[1] = whole_message ? "--message" : "--desktop";
@@ -152,17 +204,22 @@ invoke_with_gksu(const gchar *cmd, const gchar *descr, gboolean whole_message)
 gboolean
 trayapplet_create (TrayApplet *un, char *name)
 {
-        //g_print("trayicon_create()\n");
+        g_debug("trayicon_create()\n");
 
 	/* setup widgets */
-	un->tray_icon = gtk_status_icon_new_from_icon_name (name);
 	un->name = name;
+#ifdef HAVE_APP_INDICATOR
+        un->indicator = app_indicator_new (name,
+                                           name,
+                                           APP_INDICATOR_CATEGORY_SYSTEM_SERVICES);
+#else
+	un->tray_icon = gtk_status_icon_new_from_icon_name (name);
 	gtk_status_icon_set_visible (un->tray_icon, FALSE);
+#endif
 
 	return TRUE;
 }
 
-
 /* 
  the following files change:
  on "install":
@@ -253,18 +310,24 @@ gboolean file_monitor_periodic_check(gpointer data)
       // check updates
       update_check(un->update);
 
+      // run cache-changed plugin scripts
+      GDir *dir = g_dir_open(CACHE_CHANGED_PLUGIN_DIR, 0, NULL);
+      const gchar *fname, *full_path;
+      GSList *cache_changed_scripts = NULL;
+      while ( (fname = g_dir_read_name(dir)) != NULL ) {
+        full_path = g_build_filename(CACHE_CHANGED_PLUGIN_DIR, fname, NULL);
+        cache_changed_scripts = g_slist_insert_sorted(cache_changed_scripts,
+                                 (gpointer)full_path, (GCompareFunc)g_strcmp0);
+      }
+      // launch first script and queue others
+      start_next_script(0, 0, cache_changed_scripts);
+
       // any apt-get update  must be finished, otherwise 
       // apt-get install wouldn't be finished
       update_apt_is_running(un->update, FALSE);
       if(un->update_finished_timer > 0) 
 	 g_source_remove(un->update_finished_timer);
       
-      // show pending hooks/reboots
-      if(un->hook_pending) {
-	 //g_print("checking hooks now\n");
-	 check_update_hooks(un->hook);
-	 un->hook_pending = FALSE;
-      }
       if(un->reboot_pending) {
 	 //g_print("checking reboot now\n");
 	 reboot_check (un->reboot);
@@ -281,6 +344,13 @@ gboolean file_monitor_periodic_check(gpointer data)
       un->last_apt_action = 0;
    }
 
+   // show pending hooks
+   if(un->hook_pending) {
+      //g_print("checking hooks now\n");
+      check_update_hooks(un->hook);
+      un->hook_pending = FALSE;
+   }
+
    // apt-get update/install or dpkg is runing (and updates files in 
    // it's list/cache dir) or in /var/lib/dpkg/status
    if(un->apt_get_runing) 
@@ -300,13 +370,13 @@ gboolean file_monitor_periodic_check(gpointer data)
    }
 
    if(un->crashreport_pending) {
-      g_print("checking for valid crashreport now\n");
+      g_debug("checking for valid crashreport now\n");
       crashreport_check (un->crashreport);
       un->crashreport_pending = FALSE;
    }
 
    if(un->unicast_local_avahi_pending) {
-      g_print("checking for disabled avahi due to unicast .local domain now\n");
+      g_debug("checking for disabled avahi due to unicast .local domain now\n");
       avahi_disabled_check ();
       un->unicast_local_avahi_pending = FALSE;
    }
@@ -409,7 +479,7 @@ tray_icons_init(UpgradeNotifier *un)
 
    /* reboot required icon */
    un->reboot = g_new0 (TrayApplet, 1);
-   trayapplet_create(un->reboot, "reboot-notifier");
+   trayapplet_create(un->reboot, "un-reboot");
    reboot_tray_icon_init(un->reboot);
 
    /* crashreport detected icon */
@@ -427,12 +497,11 @@ system_user()
     * (see /usr/share/gdm/guest-session/guest-session-setup.sh)
     */
    int end_system_uid = 500;
-   GConfClient *gconf = gconf_client_get_default();
-   if (gconf) {
-      int i = gconf_client_get_int(gconf, GCONF_KEY_END_SYSTEM_UIDS, NULL);
-      if (i>0)
-	 end_system_uid = i;
-   }
+   GSettings *settings = g_settings_new(SETTINGS_SCHEMA);
+   int i = g_settings_get_int(settings, SETTINGS_KEY_END_SYSTEM_UIDS);
+   if (i>0)
+      end_system_uid = i;
+   g_object_unref(settings);
 
    uid_t uid = getuid();
    if (uid < end_system_uid)
@@ -440,18 +509,17 @@ system_user()
    return FALSE;
 }
 
-// this function checks if the user is in the admin group
-// if there is no admin group, we return true becuase there
-// is no way to figure if the user is a admin or not
-gboolean
-in_admin_group()
+// this function checks if the user is in the given group; return 1 if user is
+// a member, 0 if not; return 2 if group does not exist.
+static int
+user_in_group(const char* grpname)
 {
    int i, ng = 0;
    gid_t *groups = NULL;
 
-   struct group *grp = getgrnam("admin");
+   struct group *grp = getgrnam(grpname);
    if(grp == NULL) 
-      return TRUE;
+      return 2;
 
    ng = getgroups (0, NULL);
    groups = (gid_t *) malloc (ng * sizeof (gid_t));
@@ -459,31 +527,48 @@ in_admin_group()
    i = getgroups (ng, groups);
    if (i != ng) {
      free (groups);
-     return TRUE;
+     return 1;
    }
    
    for(i=0;i<ng;i++) {
       if(groups[i] == grp->gr_gid) {
         free(groups);
-        return TRUE;
+        return 1;
       }
    }
    
    if(groups != NULL)
       free(groups);
 
-   return FALSE;
+   return 0;
 }
 
+gboolean
+in_admin_group()
+{
+   int is_admin, is_sudo;
+
+   // we consider the user an admin if the user is in the "sudo" or "admin"
+   // group or neither of these groups exist
+   is_admin = user_in_group("admin");
+   is_sudo = user_in_group("sudo");
+   return is_admin == 1 || is_sudo == 1 || (is_admin + is_sudo == 4);
+}
+
+
 static GOptionEntry entries[] = 
 {
    { "debug-hooks", 0, 0, G_OPTION_ARG_NONE, &HOOK_DEBUG, "Enable hooks debugging"},
    { "debug-updates", 0, 0, G_OPTION_ARG_NONE, &UPDATE_DEBUG, "Enable updates/autolaunch debugging"},
    { "debug-inotify", 0, 0, G_OPTION_ARG_NONE, &INOTIFY_DEBUG, "Enable inotify debugging"},
-   { "debug-firmware", 0, 0, G_OPTION_ARG_NONE, &FIRMWARE_DEBUG, "Enable firmware debugging"},
+   { "debug-uevent", 0, 0, G_OPTION_ARG_NONE, &UEVENT_DEBUG, "Enable uevent debugging"},
+   { "debug-new-release-check", 0, 0, G_OPTION_ARG_NONE, &RELEASE_DEBUG, "Enable new release check debugging"},
+   { "debug-misc", 0, 0, G_OPTION_ARG_NONE, &MISC_DEBUG, "Enable uncategorized debug prints"},
    { "force", 0, 0, G_OPTION_ARG_NONE, &FORCE_START, "Force start even if the user is not in the admin group"},
    { "force-use-gksu", 0, 0, G_OPTION_ARG_NONE, &FORCE_GKSU, "Force running all commands (update-manager, synaptic) with gksu" },
    { "startup-delay", 0, 0, G_OPTION_ARG_INT, &STARTUP_DELAY, "Delay startup by given amount of seconds" },
+   { "devel-release", 0, 0, G_OPTION_ARG_NONE, &DEVEL_RELEASE, "Make the release checker check for a new development release" },
+   { "force-release-check", 0, 0, G_OPTION_ARG_NONE, &FORCE_RELEASE_CHECK, "Force release check" },
    { NULL }
 };
 
@@ -514,7 +599,11 @@ main (int argc, char **argv)
 			   debug_log_handler, NULL);
 	g_log_set_handler ("update", G_LOG_LEVEL_DEBUG,
 			   debug_log_handler, NULL);
-	g_log_set_handler ("firmware", G_LOG_LEVEL_DEBUG,
+	g_log_set_handler ("uevent", G_LOG_LEVEL_DEBUG,
+			   debug_log_handler, NULL);
+	g_log_set_handler ("release", G_LOG_LEVEL_DEBUG,
+			   debug_log_handler, NULL);
+	g_log_set_handler (NULL, G_LOG_LEVEL_DEBUG,
 			   debug_log_handler, NULL);
 
 	g_set_application_name (_("update-notifier"));
@@ -528,6 +617,13 @@ main (int argc, char **argv)
 	   exit(0);
 	}
 
+	// do not run as system user (e.g. guest user)
+	if (FORCE_RELEASE_CHECK) {
+	   GSettings *settings = g_settings_new(SETTINGS_SCHEMA);
+	   g_settings_reset(settings, SETTINGS_KEY_LAST_RELEASE_CHECK);
+	   g_object_unref(settings);
+	}
+
 	// check if it is running already
 	if (!up_get_clipboard ()) {
 	   g_warning ("already running?\n");
@@ -537,9 +633,9 @@ main (int argc, char **argv)
 	/* Create the UpgradeNotifier object */
 	un = g_new0 (UpgradeNotifier, 1);
 
-	// check for .update-notifier dir and create if needed
-	gchar *dirname = g_strdup_printf("%s/.update-notifier",
-					 g_get_home_dir());
+	// check for update-notifier dir and create if needed
+	gchar *dirname = g_strdup_printf("%s/update-notifier",
+					 g_get_user_config_dir());
 	if(!g_file_test(dirname, G_FILE_TEST_IS_DIR))
 	   g_mkdir(dirname, 0700);
 	g_free(dirname);
@@ -552,13 +648,16 @@ main (int argc, char **argv)
         // initial check for avahi
         avahi_disabled_check();
 	
-	/* setup hal so that inserted cdroms can be checked */
-	if(!up_do_hal_init(un)) {
-	   g_warning("initializing gdu failed");
+	/* setup inserted-cdroms checker */
+	if(!cdroms_init(un)) {
+	   g_warning("initializing cdroms failed");
 	}
 
-	// init missing firmware monitoring
-	missing_firmware_init();
+	// init release checker
+	release_checker_init();
+
+	// init uevent monitoring (missing firmware, etc.)
+	uevent_init();
 
 	// init gio file monitoring
 	monitor_init (un);
