@@ -13,21 +13,35 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor
+ * Boston, MA  02110-1301 USA.
  */
 
 #ifndef __UPGRADE_NOTIFIER_H__
 #define __UPGRADE_NOTIFIER_H__
 
-#include <gnome.h>
-#include <gconf/gconf.h>
-#include <gconf/gconf-client.h>
+#include <gtk/gtk.h>
+#include <glib/gi18n.h>
+
+#include "config.h"
+
+#ifdef HAVE_APP_INDICATOR
+#include <libappindicator/app-indicator.h>
+#endif
 
 #define CLIPBOARD_NAME 	"UPGRADE_NOTIFIER_SELECTION"
-#define GCONF_KEY_DEFAULT_ACTION "/apps/update-notifier/default_action"
-#define GCONF_KEY_NO_UPDATE_NOTIFICATIONS "/apps/update-notifier/no_show_notifications"
-#define GCONF_KEY_APPORT_NOTIFICATIONS "/apps/update-notifier/show_apport_crashes"
+
+#define SETTINGS_SCHEMA "com.ubuntu.update-notifier"
+#define SETTINGS_KEY_NO_UPDATE_NOTIFICATIONS "no-show-notifications"
+#define SETTINGS_KEY_APPORT_NOTIFICATIONS "show-apport-crashes"
+#define SETTINGS_KEY_END_SYSTEM_UIDS "end-system-uids"
+#define SETTINGS_KEY_AUTO_LAUNCH "auto-launch"
+#define SETTINGS_KEY_AUTO_LAUNCH_INTERVAL "regular-auto-launch-interval"
+#define SETTINGS_KEY_HIDE_REBOOT "hide-reboot-notification"
+#define SETTINGS_KEY_LAST_RELEASE_CHECK "release-check-time"
+
+#define SETTINGS_UM_SCHEMA "com.ubuntu.update-manager"
+#define SETTINGS_UM_KEY_LAST_LAUNCH "launch-time"
 
 #define HOOKS_DIR "/var/lib/update-notifier/user.d/"
 #define CRASHREPORT_HELPER "/usr/share/apport/apport-checkreports"
@@ -36,14 +50,42 @@
 #define REBOOT_FILE "/var/run/reboot-required"
 #define UNICAST_LOCAL_AVAHI_FILE "/var/run/avahi-daemon/disabled-for-unicast-local"
 
-void invoke_with_gksu(gchar *cmd, gchar *descr, gboolean whole_message);
-void invoke(gchar *cmd, gchar *desktop, gboolean with_gksu);
+// security update autolaunch minimal time (12h)
+#define AUTOLAUNCH_MINIMAL_SECURITY_INTERVAL 12*60*60
+
+// this is the age that we tolerate for updates (7 dayd)
+#define OUTDATED_NAG_AGE 60*60*24*7
+
+// this is the time we wait when we found outdated information for 
+// anacron(and friends) to update the system (2h)
+#define OUTDATED_NAG_WAIT 60*60*2
+
+// this is the script and the time to run the release-upgrades checker
+#define RELEASE_UPGRADE_CHECKER "/usr/lib/ubuntu-release-upgrader/check-new-release-gtk"
+#define RELEASE_UPGRADE_CHECK_WAIT 60*60*24*14
+
+// cache-changed plugin dir
+#define CACHE_CHANGED_PLUGIN_DIR "/usr/share/update-notifier/plugins/cache-changed"
+
+#if 0
+// testing
+#define OUTDATED_NAG_AGE 60*60
+#define OUTDATED_NAG_WAIT 6
+#endif
+
+void invoke_with_gksu(const gchar *cmd, const gchar *descr, gboolean whole_message);
+void invoke(const gchar *cmd, const gchar *desktop, gboolean with_gksu);
 gboolean in_admin_group();
 
 typedef struct _TrayApplet TrayApplet;
 struct _TrayApplet 
 {
+   GObject me;
+#ifdef HAVE_APP_INDICATOR
+   AppIndicator *indicator;
+#else
    GtkStatusIcon *tray_icon;
+#endif
    GtkWidget *menu;
    char *name;
    void *user_data;
@@ -57,8 +99,6 @@ struct _UpgradeNotifier
    TrayApplet *hook;
    TrayApplet *crashreport;
 
-   GConfClient *gconf;
-
    guint update_finished_timer; 
 
    
