@@ -8,34 +8,35 @@
 
 #include <glib.h>
 #include <glib/gstdio.h>
+#include <gio/gio.h>
 
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
 #include <stdlib.h>
-#include <dbus/dbus-glib.h>
 
 #include "update-notifier.h"
 #include "update.h"
+#include "trayappletui.h"
 
 #define UPGRADE_CHECKER PACKAGE_LIB_DIR"/update-notifier/apt-check"
 
 // command, description, desktopfile, needs_gksu
 const char* actions[][4] = {
-   { "/usr/bin/update-manager", N_("Show updates"), 
+   { "/usr/lib/update-notifier/backend_helper.py show_updates",
+     N_("Show updates"), 
      "/usr/share/applications/update-manager.desktop", 
      GINT_TO_POINTER(FALSE) },
-
-   { "/usr/sbin/synaptic --dist-upgrade-mode --non-interactive --hide-main-window -o Synaptic::AskRelated=true",
+   { "/usr/lib/update-notifier/backend_helper.py install_all_updates",
      N_("Install all updates"), "/usr/share/applications/synaptic.desktop", 
-     GINT_TO_POINTER(TRUE)
+     GINT_TO_POINTER(FALSE)
    },
-   { "/usr/sbin/synaptic --update-at-startup --non-interactive --hide-main-window", 
+   { "/usr/lib/update-notifier/backend_helper.py check_updates",
      N_("Check for updates"), "/usr/share/applications/synaptic.desktop", 
-     GINT_TO_POINTER(TRUE) },
-   { "/usr/sbin/synaptic", N_("Start package manager"), 
+     GINT_TO_POINTER(FALSE) },
+   { "/usr/lib/update-notifier/backend_helper.py start_packagemanager",
      "/usr/share/applications/synaptic.desktop", 
-     GINT_TO_POINTER(TRUE)},
+     GINT_TO_POINTER(FALSE)},
    { NULL, NULL, NULL }
 };
 
@@ -55,7 +56,7 @@ g_debug_update(const char *msg, ...)
 }
 
 static gboolean
-activate_cb (GtkWidget *widget, 
+gtk_status_icon_click_activate_cb (GtkWidget *widget, 
 	     TrayApplet *ta)
 {
    UpdateTrayAppletPrivate *priv = (UpdateTrayAppletPrivate*)ta->user_data;
@@ -70,20 +71,6 @@ activate_cb (GtkWidget *widget,
    return TRUE;
 }
 
-static gboolean
-popup_cb (GtkStatusIcon *status_icon,
-	  guint          button,
-	  guint          activate_time,
-	  TrayApplet     *un)
-{
-   gtk_menu_set_screen (GTK_MENU (un->menu),
-			gtk_status_icon_get_screen(un->tray_icon));
-   gtk_menu_popup (GTK_MENU (un->menu), NULL, NULL, 
-		   gtk_status_icon_position_menu, un->tray_icon,
-		   button, activate_time);
-   return TRUE;
-}
-
 void
 update_trayicon_update_tooltip (TrayApplet *ta, int num_upgrades)
 {
@@ -94,7 +81,7 @@ update_trayicon_update_tooltip (TrayApplet *ta, int num_upgrades)
 				      "There are %i updates available",
 				      num_upgrades),
 			     num_upgrades);
-   gtk_status_icon_set_tooltip(ta->tray_icon, updates);
+   tray_applet_ui_set_tooltip_text(ta, updates);
    g_free(updates);
 }
 
@@ -108,15 +95,14 @@ cb_action(GObject *self, void *user_data)
    invoke (actions[i][0], _(actions[i][2]), (long)actions[i][3]);
 
    UpdateTrayAppletPrivate *priv = (UpdateTrayAppletPrivate*)ta->user_data;
-   gconf_client_set_int(priv->gconf, GCONF_KEY_DEFAULT_ACTION, i, NULL);
 }
 
 void 
 cb_preferences(GObject *self, void *user_data)
 {
-   invoke_with_gksu("/usr/bin/software-properties-gtk",
-		    "/usr/share/applications/software-properties.desktop",
-		    FALSE);    
+   invoke("/usr/bin/software-properties-gtk",
+          "/usr/share/applications/software-properties.desktop",
+          FALSE);    
 }
 
 void 
@@ -126,8 +112,8 @@ cb_toggled_show_notifications(GObject *self, void *data)
       UpdateTrayAppletPrivate *priv = (UpdateTrayAppletPrivate*)ta->user_data;
 
       gboolean b = gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(self));
-      gconf_client_set_bool(priv->gconf, GCONF_KEY_NO_UPDATE_NOTIFICATIONS, 
-			    !b,NULL);
+      g_settings_set_boolean(priv->settings,
+                             SETTINGS_KEY_NO_UPDATE_NOTIFICATIONS, !b);
       
 
       NotifyNotification *n = priv->active_notification;
@@ -147,7 +133,10 @@ update_trayicon_create_menu(TrayApplet *ta)
 	ta->menu = gtk_menu_new ();
 
 	for(i=0;actions[i][0]!=NULL;i++) {
+	    if (!g_file_test(actions[i][2], G_FILE_TEST_EXISTS))
+	        continue;
 	   menuitem = gtk_menu_item_new_with_label (_(actions[i][1]));
+
 	   gtk_menu_shell_append (GTK_MENU_SHELL (ta->menu), menuitem);
 	   g_object_set_data(G_OBJECT(menuitem), "action", GINT_TO_POINTER(i));
 	   g_signal_connect(G_OBJECT(menuitem), "activate", 
@@ -159,9 +148,8 @@ update_trayicon_create_menu(TrayApplet *ta)
 
 	menuitem = gtk_check_menu_item_new_with_label(_("Show notifications"));
 	UpdateTrayAppletPrivate *priv = (UpdateTrayAppletPrivate*)ta->user_data;
-	gboolean b = gconf_client_get_bool(priv->gconf,
-					   GCONF_KEY_NO_UPDATE_NOTIFICATIONS, 
-					   NULL);
+	gboolean b = g_settings_get_boolean(priv->settings,
+	                                    SETTINGS_KEY_NO_UPDATE_NOTIFICATIONS);
 	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menuitem), !b);
 	gtk_menu_shell_append (GTK_MENU_SHELL (ta->menu), menuitem);
 	g_signal_connect(G_OBJECT(menuitem), "toggled", 
@@ -169,7 +157,8 @@ update_trayicon_create_menu(TrayApplet *ta)
 	
 
 	menuitem = gtk_image_menu_item_new_from_stock (GTK_STOCK_PREFERENCES, NULL);
-	gtk_menu_shell_append (GTK_MENU_SHELL (ta->menu), menuitem);
+	if (g_file_test("/usr/bin/software-properties-gtk", G_FILE_TEST_EXISTS))
+	    gtk_menu_shell_append (GTK_MENU_SHELL (ta->menu), menuitem);
 	g_signal_connect(G_OBJECT(menuitem), "activate", 
 			 G_CALLBACK(cb_preferences), (void*)ta);
 
@@ -190,11 +179,15 @@ update_apt_is_running(TrayApplet *ta, gboolean is_running)
 
    // if the user wants auto-launch mode, do not show the icon
    // the auto launch stuff will do its magic later
-   if(gconf_client_get_bool(priv->gconf, GCONF_KEY_AUTO_LAUNCH, NULL))
+   if(g_settings_get_boolean(priv->settings, SETTINGS_KEY_AUTO_LAUNCH))
       return;
 
    if(is_running) {
-
+#ifdef HAVE_APP_INDICATOR
+      // we can't do fancy stuff like the gray-out with app-indicator,
+      // so we just hide the tray thing here
+      tray_applet_ui_set_visible(ta, FALSE);
+#else
       // gray out the icon if apt is running, we do this only once,
       // after the first time, the storage_type changes from ICON_NAME to
       // IMAGE_PIXBUF
@@ -212,16 +205,17 @@ update_apt_is_running(TrayApplet *ta, gboolean is_running)
       }
 
       // and update the tooltip
-      gtk_status_icon_set_tooltip(ta->tray_icon, _("A package manager is working"));
+      tray_applet_ui_set_tooltip_text(ta->tray_icon, _("A package manager is working"));
       // and show it
-      gtk_status_icon_set_visible(ta->tray_icon, TRUE);
+      tray_applet_ui_set_visible(ta->tray_icon, TRUE);
+#endif
 
       if(priv->active_notification != NULL) {
 	 notify_notification_close(priv->active_notification, NULL);
 	 priv->active_notification = NULL;
       }
    } else {
-      gtk_status_icon_set_from_icon_name(ta->tray_icon, ta->name);
+      tray_applet_ui_set_icon(ta, ta->name);
    }
 }
 
@@ -238,16 +232,9 @@ show_notification(gpointer user_data)
 
    // check if the update-icon is still visible (in the delay time a 
    // update may already have been performed)
-   if(!gtk_status_icon_get_visible(ta->tray_icon))
+   if(!tray_applet_ui_get_visible(ta))
       return FALSE;
 
-   GdkRectangle area;
-   gtk_status_icon_get_geometry(ta->tray_icon, NULL, &area, NULL);
-
-   // no usefull coordiante yet, do another timeout
-   if(area.x <= 0 || area.y <= 0 || area.width <= 0 || area.height <= 0)
-      return TRUE;
-
    // now show a notification handle 
    gchar *updates;
    updates = g_strdup_printf(ngettext("There is %i update available. "
@@ -260,11 +247,10 @@ show_notification(gpointer user_data)
 				      "available updates.",
 				      priv->num_upgrades), 
 			     priv->num_upgrades);
-   NotifyNotification *n = notify_notification_new_with_status_icon(
+   NotifyNotification *n = notify_notification_new(
 			      _("Software updates available"),
 			      updates,
-			      NULL, 
-			      ta->tray_icon);
+			      NULL);
 
    GdkPixbuf* pix= gtk_icon_theme_load_icon(gtk_icon_theme_get_default(), 
 					    GTK_STOCK_DIALOG_INFO, 48,0,NULL);
@@ -283,9 +269,9 @@ show_notification(gpointer user_data)
 void 
 show_error(TrayApplet *ta, gchar *error_str)
 {
-   gtk_status_icon_set_tooltip(ta->tray_icon, error_str);
-   gtk_status_icon_set_from_icon_name(ta->tray_icon, "dialog-error");
-   gtk_status_icon_set_visible(ta->tray_icon, TRUE);
+   tray_applet_ui_set_tooltip_text(ta, error_str);
+   tray_applet_ui_set_icon(ta, "dialog-error");
+   tray_applet_ui_set_visible(ta, TRUE);
 }
 
 static gboolean
@@ -294,24 +280,18 @@ outdated_nag(TrayApplet *ta)
    struct stat buf;
    if ((stat("/var/lib/apt/periodic/update-success-stamp", &buf) == 0) &&
        (time(NULL) - buf.st_mtime > OUTDATED_NAG_AGE) ) {
-      gtk_status_icon_set_visible (ta->tray_icon, TRUE);
+      tray_applet_ui_set_visible (ta, TRUE);
       ta->name = "gtk-dialog-warning-panel";
-      GdkPixbuf *src = gtk_icon_theme_load_icon(gtk_icon_theme_get_default(),
-						ta->name, 48, 
-						GTK_ICON_LOOKUP_GENERIC_FALLBACK, 
-						NULL);
-      gtk_status_icon_set_from_pixbuf(ta->tray_icon, src);
-      g_object_unref(src);
-      gtk_status_icon_set_tooltip(ta->tray_icon,
+      tray_applet_ui_set_icon(ta, ta->name);
+      tray_applet_ui_set_tooltip_text(ta,
 				  _("The update information is outdated. "
 				    "This may be caused by network "
 				    "problems or by a repository that "
 				    "is no longer available. "
 				    "Please update manually "
-				    "by clicking on this icon and then "
-				    "selecting 'Check for updates' and "
-				    "check if some of the listed "
-				    "repositories fail."
+                                    "by selecting 'Show updates' from "
+                                    "the indicator menu, and watching "
+                                    "for any failing repositories."
 				 ));
    }
    return FALSE;
@@ -324,34 +304,47 @@ outdated_nag(TrayApplet *ta)
 static gboolean
 dpkg_lock_is_taken ()
 {
-   DBusGConnection *connection;
-   GError *error;
-   DBusGProxy *proxy;
-   gboolean locked = FALSE;
+  GError *error;
+  GVariant *answer;
+  GDBusProxy *proxy;
+  gboolean locked = FALSE;
   
-   error = NULL;
-   connection = dbus_g_bus_get (DBUS_BUS_SYSTEM, &error);
-   if (connection == NULL) {
-      g_debug_update ("Failed to open connection to bus: %s\n", error->message);
-      g_error_free (error);
-      return FALSE;
-   }
-
-  proxy = dbus_g_proxy_new_for_name (connection,
-                                     "com.ubuntu.SystemService",
-				     "/",
-				     "com.ubuntu.SystemService");
   error = NULL;
-  if (!dbus_g_proxy_call (proxy, "is_package_system_locked", &error, 
-			  G_TYPE_INVALID,
-                          G_TYPE_BOOLEAN, &locked, G_TYPE_INVALID)) {
-     g_debug_update ("error during dbus call: %s\n", error->message);
-     g_error_free (error);
-     g_object_unref (proxy);
-     return FALSE;
+  proxy = g_dbus_proxy_new_for_bus_sync (G_BUS_TYPE_SYSTEM,
+                                         G_DBUS_PROXY_FLAGS_NONE,
+                                         NULL, /* GDBusInterfaceInfo */
+                                         "com.ubuntu.SystemService",
+                                         "/",
+                                         "com.ubuntu.SystemService",
+                                         NULL, /* GCancellable */
+                                         &error);
+  if (proxy == NULL) {
+    g_debug_update ("Failed to open connection to bus: %s\n", error->message);
+    g_error_free (error);
+    return FALSE;
   }
+
+  error = NULL;
+  answer = g_dbus_proxy_call_sync (proxy, "is_package_system_locked", NULL,
+                                   G_DBUS_CALL_FLAGS_NONE, -1, NULL, &error);
   g_object_unref (proxy);
 
+  if (answer == NULL) {
+    g_debug_update ("error during dbus call: %s\n", error->message);
+    g_error_free (error);
+    return FALSE;
+  }
+
+  if (g_strcmp0 (g_variant_get_type_string (answer), "(b)") != 0) {
+    g_debug_update ("SystemService answer in unexpected format: %s\n",
+                    g_variant_get_type_string (answer));
+    g_variant_unref (answer);
+    return FALSE;
+  }
+
+  g_variant_get (answer, "(b)", &locked);
+  g_variant_unref (answer);
+
   g_debug_update ("is_package_system_locked: %i", locked);
   return locked;
 }
@@ -396,6 +389,34 @@ running_on_battery ()
 }
 #endif
 
+static gboolean
+security_updates_are_installed_unattended()
+{
+   int ret;
+   GError *error = NULL;
+
+   // run apt_check.py in "check if we install updates unattended" mode
+   // a exit status > 0 indicates that its unattended-upgrades is used
+   if ( g_file_test ("/usr/bin/unattended-upgrades", 
+		     G_FILE_TEST_IS_EXECUTABLE) ) {
+      char *cmd[] = { "/usr/bin/nice",
+		      "/usr/bin/ionice", "-c3",
+		      UPGRADE_CHECKER, 
+		      "--security-updates-unattended",	      
+		      NULL };
+
+      if (g_spawn_sync("/", cmd, NULL, 0, NULL, NULL, 
+		       NULL, NULL, &ret, &error)) {
+	 g_debug("--security-updates-unattended: %i\n", WEXITSTATUS(ret));
+	 if( WEXITSTATUS(ret) > 0 )
+	    return TRUE;
+      } else if (error) {
+	 g_print("error: %s\n", error->message);
+      }
+   }
+   return FALSE;
+}
+
 // check if the security auto launch interval is over
 // and if the user is not using auto install of security
 // updates
@@ -404,9 +425,6 @@ auto_launch_security_now(UpdateTrayAppletPrivate *priv,
 			 time_t now, 
 			 time_t last_launch)
 {
-   char *cmd[3];
-   int ret;
-   GError *error = NULL;
 
    // no security updates
    if (priv->num_security == 0)
@@ -418,23 +436,11 @@ auto_launch_security_now(UpdateTrayAppletPrivate *priv,
 		     "within the AUTOLAUNCH_MINIMAL_SECURITY_INTERVAL\n");
       return FALSE;
    }
-
-   // run apt_check.py in "check if we install updates unattended" mode
-   // a exit status > 0 indicates that its unattended-upgrades is used
-   if ( g_file_test ("/usr/bin/unattended-upgrades", 
-		     G_FILE_TEST_IS_EXECUTABLE) ) {
-      cmd[0] = UPGRADE_CHECKER;
-      cmd[1] = "--security-updates-unattended";
-      cmd[2] = NULL;
-      if (g_spawn_sync("/", cmd, NULL, 0, NULL, NULL, 
-		       NULL, NULL, &ret, &error)) {
-	 g_debug("--security-updates-unattended: %i\n", WEXITSTATUS(ret));
-	 if( WEXITSTATUS(ret) > 0 )
-	    return FALSE;
-      } else if (error) {
-	 g_print("error: %s\n", error->message);
-      }
-   }
+   
+   // if security updates are installed unattended, there is nothing
+   // to do for us
+   if (security_updates_are_installed_unattended())
+      return FALSE;
 
    g_debug_update("security updates, auto-launching");
    return TRUE;
@@ -503,20 +509,20 @@ auto_launch_now (UpdateTrayAppletPrivate *priv)
 #endif
 
    // when checking for regular updates honor the 
-   // regular_auto_launch_interval
-   interval_days = gconf_client_get_int(priv->gconf,
-					GCONF_KEY_AUTO_LAUNCH_INTERVAL, 
-					NULL);
-   g_debug_update ("interval_days from gconf: %i\n", interval_days);
+   // regular-auto-launch-interval
+   interval_days = g_settings_get_int(priv->settings,
+					SETTINGS_KEY_AUTO_LAUNCH_INTERVAL);
+   g_debug_update ("interval_days: %i\n", interval_days);
 
    if (interval_days <= 0) 
       return TRUE;
 
    // check last launch time 
-   last_launch = gconf_client_get_int(priv->gconf,
-				      GCONF_KEY_LAST_LAUNCH,
-				      NULL);
-   g_debug_update ("last_launch from gconf: %i (%s)\n", last_launch, ctime(&last_launch));
+   GSettings *um_settings = g_settings_new(SETTINGS_UM_SCHEMA);
+   last_launch = g_settings_get_int(um_settings,
+                                    SETTINGS_UM_KEY_LAST_LAUNCH);
+   g_debug_update ("last_launch: %i (%s)\n", last_launch, ctime(&last_launch));
+   g_object_unref(um_settings);
 
    time_t now = time(NULL);
    if (auto_launch_security_now(priv, now, last_launch))
@@ -546,9 +552,12 @@ update_check (TrayApplet *ta)
    priv->num_upgrades = 0;
    priv->num_security = 0;
    int exit_status = 0;
-   GError *error;
+   GError *error = NULL;
 
-   char *cmd[] = { UPGRADE_CHECKER, NULL };
+   char *cmd[] = { "/usr/bin/nice",
+		   "/usr/bin/ionice", "-c3",
+		   UPGRADE_CHECKER, 
+		   NULL };
    char *ret;
 
    if(g_spawn_sync(NULL, cmd, NULL,  
@@ -573,7 +582,7 @@ update_check (TrayApplet *ta)
 				      "Package Manager from the "
 				      "right-click menu or apt-get in "
 				      "a terminal to see what is wrong.\n"
-				      "The error message was: '%s'"),
+				      "The error message was: '%s'. "),
 				    &ret[2]);
 	 else
 	    g_string_append(error, _("An error occurred, please run "
@@ -615,8 +624,7 @@ update_check (TrayApplet *ta)
 	  (time(NULL) - buf.st_mtime > OUTDATED_NAG_AGE) ) 
 	 g_timeout_add_seconds(OUTDATED_NAG_WAIT, (GSourceFunc)outdated_nag, ta);
 
-      gtk_status_icon_set_visible (ta->tray_icon, FALSE);
-      
+      tray_applet_ui_set_visible (ta, FALSE);
       if(priv->active_notification != NULL) {
 	 notify_notification_close(priv->active_notification, NULL);
 	 priv->active_notification = NULL; 
@@ -628,35 +636,34 @@ update_check (TrayApplet *ta)
       ta->name = "software-update-available";
    else
       ta->name = "software-update-urgent";
-   gtk_status_icon_set_from_icon_name(ta->tray_icon, ta->name);
+   tray_applet_ui_set_icon(ta, ta->name);
 
    // update the tooltip
    update_trayicon_update_tooltip(ta, priv->num_upgrades);
 
    // check if the user wants to see the icon or launch
    // the default action
-   if(gconf_client_get_bool(priv->gconf,
-			    GCONF_KEY_AUTO_LAUNCH, NULL)) 
+   if(g_settings_get_boolean(priv->settings, SETTINGS_KEY_AUTO_LAUNCH))
    {
-      gtk_status_icon_set_visible(ta->tray_icon, FALSE);
+      tray_applet_ui_set_visible(ta, FALSE);
       if (auto_launch_now(priv)) 
       {
 	 g_spawn_command_line_async("nice ionice -c3 update-manager "
-				    "--no-focus-on-map", NULL);
+				    "--no-update --no-focus-on-map", NULL);
       }
       return TRUE;
    }
 
    // if we are already visible, skip the rest
-   if(gtk_status_icon_get_visible (ta->tray_icon))
+   if(tray_applet_ui_get_visible (ta))
       return TRUE;
 
    // show the icon
-   gtk_status_icon_set_visible(ta->tray_icon, TRUE);
+   tray_applet_ui_set_visible(ta, TRUE);
    
    // the user does not no notification messages
-   if(gconf_client_get_bool(priv->gconf,
-			    GCONF_KEY_NO_UPDATE_NOTIFICATIONS, NULL))
+   if(g_settings_get_boolean(priv->settings,
+                             SETTINGS_KEY_NO_UPDATE_NOTIFICATIONS))
       return TRUE;
 
    // show the notification with some delay. otherwise on a login
@@ -672,23 +679,22 @@ update_tray_icon_init(TrayApplet *ta)
 {
    // create the private data struct
    UpdateTrayAppletPrivate *priv = g_new0(UpdateTrayAppletPrivate, 1);
-   priv->gconf = gconf_client_get_default();
+   priv->settings = g_settings_new(SETTINGS_SCHEMA);
    priv->apt_is_running = FALSE;
    priv->active_notification = NULL;
    ta->user_data = priv;
 
-   // connect the signals we need
+#ifndef HAVE_APP_INDICATOR
+   // the default action for the gtk_status_icon
    g_signal_connect (G_OBJECT(ta->tray_icon),
 		     "activate",
-		     G_CALLBACK (activate_cb),
-		     ta);
-   g_signal_connect (G_OBJECT(ta->tray_icon),
-		     "popup-menu",
-		     G_CALLBACK (popup_cb),
+		     G_CALLBACK (gtk_status_icon_click_activate_cb),
 		     ta);
+#endif
 
    /* Menu initialization */
    update_trayicon_create_menu (ta);
+   tray_applet_ui_set_menu (ta, ta->menu);
    
    /* Check for updates for the first time */
    update_check (ta);
